/* 
 * net2280dis - disassamble net2280 eeprom
 *
 * $Id: decode.c 3 2004-12-13 23:34:00Z bene $
 * 
 * Copyright (C) 2004 Benedikt Spranger
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 * 
 */

#include <stdio.h>

#include "net2280tbl.h"

extern int verbose, pass;
char label [0xFFFF];

void direct (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (pass == 2)
	{
		if (data [1] < 0x80) printf ("0x%02x", data [1]);
		else printf ("%s", sfr [data [1] - 0x80]);
		if (suf) printf ("%s", suf);
	}
	return;
}

void bitrel (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	bit (pc, 1, data, NULL);
	if (pass == 2) printf (", ");
	relative (pc + 1, 1, data + 1, suf);
}

void dirdata (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	direct (pc, 1, data, NULL);
	if (pass == 2) printf (", ");
	data8 (pc, 1, data + 1, suf);
}

void bit (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (pass == 2)
	{
		int b, c;
		
		b = data [1] & 0x03;
		c = data [1] & 0xF4;
		
		printf ("data = %02x\nc = %02x\nb = %d\n", data [1], c, b);
		
		if (c < 0x80) printf ("0x%02x.%dXXXX", c, b);
		else printf ("%s.%dXXXX", sfr [c - 0x80], b);
		if (suf) printf ("%s", suf);
	}
}

void dirdir (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	direct (pc, 1, data, NULL);
	if (pass == 2) printf (", ");
	direct (pc, 1, data + 1, suf);
}

void datarel (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	data8 (pc, 1, data, NULL);
	if (pass == 2) printf (", ");
	relative (pc + 1, 1, data + 1, suf);
}

void dirrel (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	direct (pc, 1, data, NULL);
	if (pass == 2) printf (", ");
	relative (pc + 1, 1, data + 1, suf);
}

void relative (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	switch (pass) 
	{
	case 0: break;
	case 1: 
		label [pc + 2 + (signed char) data [1]] = 1;
		break;
	case 2:
		printf ("%04x", pc + 2 + (signed char) data [1]);
		if (suf) printf ("%s", suf);
		break;
	default:
		fprintf (stderr, "unknown pass %d\n", pass);
		break;
	}
	
	return;
}

void nop (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (suf) printf ("%s", suf);
	return;
}

void jmps (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	unsigned int pcold = pc;
	
	if ((*data & 0x0F) == 1) 
	{
		pc +=2;
		pc &= 0xE000;
		pc |= ((*data & 0xE) >> 5)*256 + data [1];
	}
	else pc = 256*data [1] + data [2];
	
	switch (pass)
	{
	case 0: break;
	case 1: 
		if (!(*data & 0x10)) label [pcold] = 1;
		label [pc] = 1;
		break;
	case 2:
		if (*data & 0x10) printf ("%04x", pc);
		else printf ("%04x\n;", pc);
		if (suf) printf ("%s", suf);
		break;
	default: 
		fprintf (stderr, "unknown pass %d\n", pass);
		break;
	}
	
	return;
}

void ret (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (pass == 2) 
	{
		printf ("\n;");
		if (suf) printf ("%s", suf);
	}
	
	return;
}

void data16 (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (pass == 2) 
	{
		printf (" #0x%04x", 256*data [1] + data [2]);
		if (suf) printf ("%s", suf);
	}
	
	return;
}

void data8 (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	if (pass == 2) 
	{
		printf (" #0x%02x", data [1]);
		if (suf) printf ("%s", suf);
	}
	
	return;
}

void unknown (unsigned int pc, unsigned int size, unsigned char *data, char *suf)
{
	int i;
	if (pass == 2) 
	{
		printf ("unknown parameter:");
		for (i = 1; i <= size; i++) printf (" %02x", data [i]);
	}
	
	return;
}

void net2280dis (unsigned int prog_size, unsigned char *prog)
{
	unsigned int i, p, pold, v;
	char *f, c;
	
	printf ("\n8051 disassambled code:\n");
	
	for (pass = 1; pass < 3; pass++)
	{
		for (p = 0; p < prog_size; p++)
		{
			switch (pass)
			{
			case 0: break;
			case 1: break;
			case 2:
				if (verbose) 
				{
					printf ("\n%04x: ", p);
					for (i = 0; 
					     i <= opc [prog [p]].size; i++)
					    printf ("%02x ", prog [p + i]);
					printf ("\t");
					for (i = 0; 
					     i <= opc [prog [p]].size; i++)
					{
						c = prog [p + i];
						printf ("%c", (isprint (c)) ? 
							c : '.');
					}
				}
				
				if (label [p]) printf ("\n%04x:\t", p);
				else printf ("\n\t");
				
				printf ("%s", opc [prog [p]].desc);
				break;
			default:
				fprintf (stderr, "unknown pass %d\n", pass);
				break;
			}
			
			opc [prog [p]].f (p, opc [prog [p]].size, prog + p,
					  opc [prog [p]].suf);
			p += opc [prog [p]].size;
		}
	}
}
