/* 
 * net2280dis - disassamble net2280 eeprom
 * 
 * $Id: net2280dis.c 3 2004-12-13 23:34:00Z bene $
 * 
 * Copyright (C) 2004 Benedikt Spranger
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <getopt.h>
#include <error.h>
#include "system.h"

#include "cmdline.h"
#include "net2280dis.h"

char *xmalloc ();
char *xrealloc ();
char *xstrdup ();

int verbose;
int pass;

int
main(int argc, char **argv)
{
	FILE *fhan;
	net2280_header_t header;
	net2280_confreg_t *regs;
	unsigned int reg_size, reg_num, prog_size;
	unsigned char tmp [2];
	unsigned char *prog;
	
	Cmdline *cmd = parseCmdline (argc, argv);
	
	if (cmd->show_helpP) usage ();
	
	if (cmd->show_versionP)
	{
		printf ("%s %s\n", argv[0], VERSION);
		return EXIT_SUCCESS;
	}
	
	verbose = cmd->verboseP;
	
	fhan = fopen (argv [1], "rb");
	if (!fhan) 
	{
		perror ("could not open file");
		return EXIT_FAILURE;
	}
	
	if (fread ((void *) &header, 1, 4, fhan) != 4)
	{
		fprintf (stderr, "could not read header\n");
		fclose (fhan);
		return EXIT_FAILURE;
	}
	
	printf ("Header:\n");
	printf ("signature = %02x\n", header.sig);
	printf ("format    = %02x\n", header.format);
	printf ("lsb       = %02x\n", header.lsb);
	printf ("msb       = %02x\n\n", header.msb);
	
	printf ("EEPROM is %s\n", 
		(header.sig  == 0x5a) ? "valid" : "NOT VALID");
	printf ("Configuration Register load: %s\n", 
		(header.format & 1) ? "yes" : "no");
	printf ("Programm Memory load       : %s\n", 
		(header.format & 2) ? "yes" : "no");
	printf ("8051 Start Enable          : %s\n", 
		(header.format & 4) ? "yes" : "no");
	
	reg_size = header.lsb + 256*header.msb;
	reg_num = reg_size/sizeof (net2280_confreg_t);
	
	printf ("Configuration Register Size: %5d bytes\n", reg_size);
	
	regs = calloc (reg_size, 1);
	if (!regs)
	{
		fprintf (stderr, "could not allocate register memory\n");
		fclose (fhan);
		return EXIT_FAILURE;
	}
	
	if (fread ((void *) regs, 1, reg_size, fhan) != reg_size)
	{
		fprintf (stderr, "could not read registers\n");
		fclose (fhan);
		return EXIT_FAILURE;
	}
	
	if (fread ((void *) tmp, 1, 2, fhan) != 2)
	{
		fprintf (stderr, "could not read prog size\n");
		fclose (fhan);
		return EXIT_FAILURE;
	}
	
	prog_size = tmp [0] + 256*tmp [1];
	
	printf ("8051 Programm Size         : %5d bytes\n", prog_size);
	
	prog = calloc (prog_size, 1);
	if (!prog)
	{
		fprintf (stderr, "could not allocate programm memory\n");
		fclose (fhan);
		return EXIT_FAILURE;
	}
	
	fread ((void *) prog, 1, prog_size, fhan);
	
	fclose (fhan);
	
	print_regs (reg_num, regs);
	
//	mem_dump (prog_size, prog);
	net2280dis (prog_size, prog);
	
	return EXIT_SUCCESS;
}

