/* 
 * net2280dis - disassamble net2280 eeprom
 *
 * $Id: net2280tbl.h 3 2004-12-13 23:34:00Z bene $
 * 
 * Copyright (C) 2004 Benedikt Spranger
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 * 
 */

#ifndef NET2280TBL_H
# define NET2280TBL_H

void unknown (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void nop (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void data8 (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void data16 (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void jmps (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void ret (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void direct (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void relative (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void bitrel (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void dirdata (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void bit (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void dirdir (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void datarel (unsigned int pc, unsigned int size, unsigned char *data, char *suf);
void dirrel (unsigned int pc, unsigned int size, unsigned char *data, char *suf);

typedef void (*opcode_funct_t)(unsigned int, unsigned int, unsigned char *, char *);

typedef struct opcode 
{
	char		*desc;
	unsigned int	size;
	opcode_funct_t	f;
	char 		*suf;
} opcode_t;

/* Names for Special Function Registers */

char *sfr [] = 
{
	"p0",	"sp",	"dpl",	"dph",		/* 80 - 83 */
	"dpl1",	"dph1",	"dps",	"pcon",		/* 84 - 87 */
	"tcon",	"tmod",	"tl0",	"tl1",		/* 88 - 8b */
	"th0",	"th1",	"ckcon","0x8f",		/* 8c - 8f */
	"p1",	"0x91",	"0x92",	"0x93",		/* 90 - 93 */
	"0x94",	"0x95",	"0x96",	"0x97",		/* 94 - 97 */
	"scon",	"sbuf",	"0x9a",	"0x9b",		/* 98 - 9b */
	"0x9c",	"0x9d",	"0x9e",	"0x9f",		/* 9c - 9f */
	"p2",	"0xa1",	"0xa2",	"0xa3",		/* a0 - a3 */
	"0xa4",	"0xa5",	"0xa6",	"0xa7",		/* a4 - a7 */
	"ie",	"saddr","0xaa",	"0xab",		/* a8 - ab */
	"0xac",	"0xad",	"0xae",	"0xaf",		/* ac - af */
	"p3",	"0xb1",	"0xb2",	"0xb3",		/* b0 - b3 */
	"0xb4",	"0xb5",	"0xb6",	"0xb7",		/* b4 - b7 */
	"ip",	"saden","0xba",	"0xbb",		/* b8 - bb */
	"0xbc",	"0xbd",	"0xbe",	"0xbf",		/* bc - bf */
	"0xc0",	"0xc1",	"0xc2",	"0xc3",		/* c0 - c3 */
	"pmr",	"0xc5",	"0xc6",	"0xc7",		/* c4 - c7 */
	"t2con","t2mod","rcap2l","rcap2h",	/* c8 - cb */
	"tl2",	"th2",	"0xce",	"0xcf",		/* cc - cf */
	"psw",	"0xd1",	"0xd2",	"0xd3",		/* d0 - d3 */
	"0xd4",	"0xd5",	"0xd6",	"0xd7",		/* d4 - d7 */
	"0xd8",	"0xd9",	"0xda",	"0xdb",		/* d8 - db */
	"0xdc",	"0xdd",	"0xde",	"0xdf",		/* dc - df */
	"acc",	"0xe1",	"0xe2",	"0xe3",		/* e0 - e3 */
	"0xe4",	"0xe5",	"0xe6",	"0xe7",		/* e4 - e7 */
	"irq0a","0xe9",	"0xea",	"0xeb",		/* e8 - eb */
	"0xec",	"0xed",	"0xee",	"0xef",		/* ec - ef */
	"b",	"0xf1",	"cfgctl0","0xf3",	/* f0 - f3 */
	"cfgaddr0","cfgaddr1","0xf6","0xf7",	/* f4 - f7 */
	"irq0b","cfgdata0","cfgdata1","cfgdata2",/* f8 - fb */
	"cfgdata3","0xfd","0xfe","0xff"		/* fc - ff */
};

const opcode_t opc [] = 
{
	/* 00 - 0F */
	{"nop ", 0, nop, NULL},      
	{"ajmp ", 1, jmps, NULL},     
	{"ljmp ", 2, jmps, NULL},     
	{"rr a", 0, nop, NULL},
	{"inc a", 0, nop, NULL},     
	{"inc ", 1, direct, NULL},      
	{"inc @r0", 0, nop, NULL},   
	{"inc @r1", 0, nop, NULL},   
	{"inc r0", 0, nop, NULL},    
	{"inc r1", 0, nop, NULL},    
	{"inc r2", 0, nop, NULL},    
	{"inc r3", 0, nop, NULL},    
	{"inc r4", 0, nop, NULL},    
	{"inc r5", 0, nop, NULL},    
	{"inc r6", 0, nop, NULL},
	{"inc r7", 0, nop, NULL},
	/* 10 - 1F */
	{"jbc ", 2, bitrel, NULL},      
	{"acall ", 1, jmps, NULL},    
	{"lcall ", 2, jmps, NULL},    
	{"rrc a", 0, nop, NULL},     
	{"dec a", 0, nop, NULL},     
	{"dec ", 0, nop, NULL},      
	{"dec @r0", 0, nop, NULL},   
	{"dec @r1", 0, nop, NULL},   
	{"dec r0", 0, nop, NULL},    
	{"dec r1", 0, nop, NULL},    
	{"dec r2", 0, nop, NULL},    
	{"dec r3", 0, nop, NULL},    
	{"dec r4", 0, nop, NULL},    
	{"dec r5", 0, nop, NULL},    
	{"dec r6", 0, nop, NULL},
	{"dec r7", 0, nop, NULL}, 
	/* 20 - 2F */
	{"jb ", 2, bitrel, NULL},       
	{"ajmp ", 1, jmps, NULL},     
	{"ret ", 0, ret, NULL},      
	{"rl a", 0, nop, NULL},      
	{"add a,", 1, data8, NULL},   
	{"add a,", 1, direct, NULL},    
	{"add a, @r0", 0, nop, NULL}, 
	{"add a, @r1", 0, nop, NULL}, 
	{"add a, r0", 0, nop, NULL},  
	{"add a, r1", 0, nop, NULL},  
	{"add a, r2", 0, nop, NULL},  
	{"add a, r3", 0, nop, NULL},  
	{"add a, r4", 0, nop, NULL},  
	{"add a, r5", 0, nop, NULL},  
	{"add a, r6", 0, nop, NULL},  
	{"add a, r7", 0, nop, NULL},
	/* 30 - 3F */
	{"jnb ", 2, bitrel, NULL},      
	{"acall ", 1, jmps, NULL},    
	{"reti ", 0, ret, NULL},     
	{"rlc a", 0, nop, NULL},     
	{"addc a,", 1, data8, NULL},  
	{"addc a,", 1, direct, NULL},   
	{"addc a, @r0", 0, nop, NULL},
	{"addc a, @r1", 0, nop, NULL},
	{"addc a, r0", 0, nop, NULL}, 
	{"addc a, r1", 0, nop, NULL}, 
	{"addc a, r2", 0, nop, NULL}, 
	{"addc a, r3", 0, nop, NULL}, 
	{"addc a, r4", 0, nop, NULL}, 
	{"addc a, r5", 0, nop, NULL}, 
	{"addc a, r6", 0, nop, NULL}, 
	{"addc a, r7", 0, nop, NULL},
	/* 40 - 4F */
	{"jc ", 1, relative, NULL},       
	{"ajmp ", 1, jmps, NULL},     
	{"orl ", 1, direct, ", a"},      
	{"orl ", 1, dirdata, NULL},      
	{"orl a,", 1, data8, NULL},   
	{"orl a,", 1, direct, NULL},    
	{"orl a, @r0", 0, nop, NULL}, 
	{"orl a, @r1", 0, nop, NULL}, 
	{"orl a, r0", 0, nop, NULL},  
	{"orl a, r1", 0, nop, NULL},  
	{"orl a, r2", 0, nop, NULL},  
	{"orl a, r3", 0, nop, NULL},  
	{"orl a, r4", 0, nop, NULL},  
	{"orl a, r5", 0, nop, NULL},  
	{"orl a, r6", 0, nop, NULL},  
	{"orl a, r7", 0, nop, NULL},  
	/* 50 - 5F */
	{"jnc ", 1, relative, NULL},      
	{"acall ", 1, jmps, NULL},    
	{"anl ", 1, direct, ", a"},      
	{"anl ", 2, dirdata, NULL},      
	{"anl a,", 1, data8, NULL},   
	{"anl a,", 1, direct, NULL},    
	{"anl a, @r0", 0, nop, NULL}, 
	{"anl a, @r1", 0, nop, NULL}, 
	{"anl a, r0", 0, nop, NULL},  
	{"anl a, r1", 0, nop, NULL},  
	{"anl a, r2", 0, nop, NULL},  
	{"anl a, r3", 0, nop, NULL},  
	{"anl a, r4", 0, nop, NULL},  
	{"anl a, r5", 0, nop, NULL},  
	{"anl a, r6", 0, nop, NULL},  
	{"anl a, r7", 0, nop, NULL},  
	/* 60 - 6F */
	{"jz ", 1, relative, NULL},       
	{"ajmp ", 1, jmps, NULL},     
	{"xrl ", 1, direct, ",a"},      
	{"xrl ", 1, dirdata, NULL},      
	{"xrl a,", 1, data8, NULL},   
	{"xrl a,", 1, direct, NULL},    
	{"xrl a, @r0", 0, nop, NULL}, 
	{"xrl a, @r1", 0, nop, NULL}, 
	{"xrl a, r0", 0, nop, NULL},  
	{"xrl a, r1", 0, nop, NULL},  
	{"xrl a, r2", 0, nop, NULL},  
	{"xrl a, r3", 0, nop, NULL},  
	{"xrl a, r4", 0, nop, NULL},  
	{"xrl a, r5", 0, nop, NULL},  
	{"xrl a, r6", 0, nop, NULL},  
	{"xrl a, r7", 0, nop, NULL},  
	/* 70 - 7F */
	{"jnz ", 1, relative, NULL},      
	{"acall ", 1, jmps, NULL},    
	{"orl c,", 1, bit, NULL},    
	{"jmp @a+dptr", 0, nop, "\n;"},
	{"mov a,", 1, data8, NULL},   
	{"mov ", 1, dirdata, NULL},      
	{"mov @r0,", 1, data8, NULL}, 
	{"mov @r1,", 1, data8, NULL}, 
	{"mov r0,", 1, data8, NULL},  
	{"mov r1,", 1, data8, NULL},  
	{"mov r2,", 1, data8, NULL},  
	{"mov r3,", 1, data8, NULL},  
	{"mov r4,", 1, data8, NULL},  
	{"mov r5,", 1, data8, NULL},  
	{"mov r6,", 1, data8, NULL},
	{"mov r7,", 1, data8, NULL},  
	/* 80 - 8F */
	{"sjmp ", 1, jmps, NULL},     
	{"ajmp ", 1, jmps, NULL},     
	{"anl c,", 1, bit, NULL},    
	{"movc a,@a+pc", 0, nop, NULL},
	{"div ab", 0, nop, NULL},    
	{"mov ", 2, dirdir, NULL},      
	{"mov ", 1, direct, ", @R0"},      
	{"mov ", 1, direct, ", @R1"},      
	{"mov ", 1, direct, ", R0"},      
	{"mov ", 1, direct, ", R1"},      
	{"mov ", 1, direct, ", R2"},      
	{"mov ", 1, direct, ", R3"},      
	{"mov ", 1, direct, ", R4"},      
	{"mov ", 1, direct, ", R5"},      
	{"mov ", 1, direct, ", R6"},      
	{"mov ", 1, direct, ", R7"},     
	/* 90 - 9F */
	{"mov dptr,", 2, data16, NULL},
	{"acall ", 1, jmps, NULL},    
	{"mov ", 1, bit, ", c"},      
	{"movc a,@a+dptr", 0, nop, NULL},
	{"subb a,", 1, data8, NULL},  
	{"subb a,", 1, direct, NULL},   
	{"subb a, @r0", 0, nop, NULL},
	{"subb a, @r1", 0, nop, NULL},
	{"subb a, r0", 0, nop, NULL}, 
	{"subb a, r1", 0, nop, NULL}, 
	{"subb a, r2", 0, nop, NULL}, 
	{"subb a, r3", 0, nop, NULL}, 
	{"subb a, r4", 0, nop, NULL}, 
	{"subb a, r5", 0, nop, NULL}, 
	{"subb a, r6", 0, nop, NULL}, 
	{"subb a, r7", 0, nop, NULL}, 
	/* a0 - aF */
	{"orl c, ", 1, bit, NULL},   
	{"ajmp ", 1, jmps, NULL},     
	{"mov c,", 1, bit, NULL},    
	{"inc dptr", 0, nop, NULL},  	
	{"mul ab", 0, nop, NULL},    
	{"db ", 0, nop, NULL},       
	{"mov @r0,", 1, direct, NULL},  
	{"mov @r1,", 1, direct, NULL},  
	{"mov r0,", 1, direct, NULL},   
	{"mov r1,", 1, direct, NULL},   
	{"mov r2,", 1, direct, NULL},   
	{"mov r3,", 1, direct, NULL},   
	{"mov r4,", 1, direct, NULL},   
	{"mov r5,", 1, direct, NULL},   
	{"mov r6,", 1, direct, NULL},   
	{"mov r7,", 1, direct, NULL},  
	/* B0 - BF */
	{"anl c,", 1, bit, NULL},   
	{"acall ", 1, jmps, NULL},    
	{"cpl ", 1, bit, NULL},      
	{"cpl c", 0, nop, NULL},     
	{"cjne a,", 2, datarel, NULL},  
	{"cjne a,", 2, dirrel, NULL},   
	{"cjne @r0,", 2, datarel, NULL},
	{"cjne @r1,", 2, datarel, NULL},
	{"cjne r0,", 2, datarel, NULL}, 
	{"cjne r1,", 2, datarel, NULL}, 
	{"cjne r2,", 2, datarel, NULL},
	{"cjne r3,", 2, datarel, NULL}, 
	{"cjne r4,", 2, datarel, NULL}, 
	{"cjne r5,", 2, datarel, NULL}, 
	{"cjne r6,", 2, datarel, NULL}, 
	{"cjne r7,", 2, datarel, NULL}, 
	/* C0 - CF */
	{"push ", 1, direct, NULL},     
	{"ajmp ", 1, jmps, NULL},     
	{"clr ", 1, bit, NULL},      
	{"clr c", 0, nop, NULL},     
	{"swap a", 0, nop, NULL},    
	{"xch a,", 1, direct, NULL},    
	{"xch a, @r0", 0, nop, NULL}, 
	{"xch a, @r1", 0, nop, NULL}, 
	{"xch a, r0", 0, nop, NULL},  
	{"xch a, r1", 0, nop, NULL},  
	{"xch a, r2", 0, nop, NULL},  
	{"xch a, r3", 0, nop, NULL},  
	{"xch a, r4", 0, nop, NULL},  
	{"xch a, r5", 0, nop, NULL},  
	{"xch a, r6", 0, nop, NULL},  
	{"xch a, r7", 0, nop, NULL}, 
	/* D0 - DF */
	{"pop ", 1, direct, NULL},      
	{"acall ", 1, jmps, NULL},    
	{"setb ", 1, bit, NULL},     
	{"setb c", 0, nop, NULL},    
	{"da a", 0, nop, NULL},      
	{"djnz ", 1, dirrel, NULL},     
	{"xchd a, @r0", 0, nop, NULL},
	{"xchd a, @r1", 0, nop, NULL},
	{"djnz r0,", 1, relative, NULL},  
	{"djnz r1,", 1, relative, NULL},  
	{"djnz r2,", 1, relative, NULL},  
	{"djnz r3,", 1, relative, NULL},  
	{"djnz r4,", 1, relative, NULL},  
	{"djnz r5,", 1, relative, NULL},  
	{"djnz r6,", 1, relative, NULL},  
	{"djnz r7,", 1, relative, NULL},  
	/* E0 - EF */
	{"movx a, @dptr", 0, nop, NULL},
	{"ajmp ", 1, jmps, NULL},   
	{"movx a,@r0", 0, nop, NULL},
	{"movx a,@r1", 0, nop, NULL},
	{"clr a", 0, nop, NULL},     
	{"mov a,", 1, direct, NULL},    
	{"mov a, @r0", 0, nop, NULL}, 
	{"mov a, @r1", 0, nop, NULL}, 
	{"mov a, r0", 0, nop, NULL},  
	{"mov a, r1", 0, nop, NULL},  
	{"mov a, r2", 0, nop, NULL},  
	{"mov a, r3", 0, nop, NULL},  
	{"mov a, r4", 0, nop, NULL},  
	{"mov a, r5", 0, nop, NULL},  
	{"mov a, r6", 0, nop, NULL},  
	{"mov a, r7", 0, nop, NULL},
	/* F0 - FF */
	{"movx @dptr,a", 0, nop, NULL},
	{"acall ", 1, jmps, NULL},  
	{"movx @r0, a", 0, nop, NULL},
	{"movx @r1, a", 0, nop, NULL},
	{"cpl a", 0, nop, NULL},     
	{"mov ", 1, direct, ", a"},      
	{"mov @r0, a", 0, nop, NULL}, 
	{"mov @r1, a", 0, nop, NULL}, 
	{"mov r0, a", 0, nop, NULL},  
	{"mov r1, a", 0, nop, NULL},  
	{"mov r2, a", 0, nop, NULL},  
	{"mov r3, a", 0, nop, NULL},  
	{"mov r4, a", 0, nop, NULL},  
	{"mov r5, a", 0, nop, NULL},  
	{"mov r6, a", 0, nop, NULL},  
	{"mov r7, a", 0, nop, NULL}   
};

#endif /* NET2280TBL_H */
